package ch.epfl.ppo;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public final class StemAndLeafBiPlot implements Observer {
    private final List<Cell> cellsLeft, cellsRight;

    public StemAndLeafBiPlot(List<Cell> cellsLeft, List<Cell> cellsRight) {
        this.cellsLeft = new ArrayList<>(cellsLeft);
        this.cellsRight = new ArrayList<>(cellsRight);

        for (Cell c: cellsLeft)
            c.addObserver(this);
        for (Cell c: cellsRight)
            c.addObserver(this);
        drawPlot();
    }

    @Override
    public void update(Subject cell) {
        drawPlot();
    }

    private void drawPlot() {
        Map<Integer, List<Integer>> stemsL = stemsFor(cellsLeft);
        Map<Integer, List<Integer>> stemsR = stemsFor(cellsRight);
        int minStem = Math.min(Collections.min(stemsL.keySet()), Collections.min(stemsR.keySet()));
        int maxStem = Math.max(Collections.max(stemsL.keySet()), Collections.max(stemsR.keySet()));
        System.out.println();
        for (int stem = minStem; stem <= maxStem; ++stem) {
            String leavesL = leavesForStem(stem, stemsL, true);
            String leavesR = leavesForStem(stem, stemsR, false);
            System.out.println(String.format("%50s|%1d|%s", leavesL, stem, leavesR));
        }
    }

    private String leavesForStem(int stem, Map<Integer, List<Integer>> stems, boolean reversed) {
        List<Integer> leaves = stems.containsKey(stem) ? stems.get(stem) : Collections.<Integer>emptyList();
        Collections.sort(leaves);
        StringBuilder leavesB = new StringBuilder();
        for (int l: leaves)
            leavesB.append(l);
        if (reversed)
            leavesB.reverse();
        return leavesB.toString();
    }

    private Map<Integer, List<Integer>> stemsFor(List<Cell> cells) {
        Map<Integer, List<Integer>> stems = new HashMap<>();
        for (Cell c: cells) {
            int stem = c.value() / 10;
            int leaf = c.value() % 10;
            if (!stems.containsKey(stem))
                stems.put(stem, new ArrayList<Integer>());
            stems.get(stem).add(leaf);
        }
        return stems;
    }
}
