package ch.epfl.ppo;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public final class StemAndLeafPlot implements Observer {
    private final List<Cell> cells;

    public StemAndLeafPlot(List<Cell> cells) {
        this.cells = new ArrayList<>(cells);

        for (Cell c: cells)
            c.addObserver(this);
        drawPlot();
    }

    @Override
    public void update(Subject cell) {
        drawPlot();
    }

    private void drawPlot() {
        Map<Integer, List<Integer>> stems = new HashMap<>();
        for (Cell c: cells) {
            int stem = c.value() / 10;
            int leaf = c.value() % 10;
            if (!stems.containsKey(stem))
                stems.put(stem, new ArrayList<Integer>());
            stems.get(stem).add(leaf);
        }
        int minStem = Collections.min(stems.keySet());
        int maxStem = Collections.max(stems.keySet());
        System.out.println();
        for (int stem = minStem; stem <= maxStem; ++stem) {
            List<Integer> leaves = stems.containsKey(stem) ? stems.get(stem) : Collections.<Integer>emptyList();
            Collections.sort(leaves);
            StringBuilder leavesB = new StringBuilder();
            for (int l: leaves)
                leavesB.append(l);
            System.out.println(String.format("%3d|%s", stem, leavesB));
        }
    }
}
